/* $Id: info.C,v 1.2 2002/05/17 09:21:13 schmitzj Exp $ */

#include <stdio.h> 
#include <stdarg.h> 

#define EDONKEYTEMPPATH "/path/to/your/donkey/temp/" 

bool error(const char* format, ...) { 
va_list marker; 
va_start(marker, format); 
vprintf(format, marker); 
va_end(marker); 
return false; 
} 

const char* priority(int value) { 
switch (value) { 
case 0: 
return "Low"; 
case 1: 
return "Normal"; 
case 2: 
return "High"; 
default: 
static char buffer[128]; 
sprintf(buffer, "unknown (%d)", value); 
return buffer; 
} 
} 

const char* status(int value) { 
switch (value) { 
case 0: 
return "Looking..."; 
case 1: 
return "Paused"; 
default: 
static char buffer[128]; 
sprintf(buffer, "unknown (%d)", value); 
return buffer; 
} 
} 

bool partInfo(FILE* file) { 
unsigned char version; 
if (fread(&version, 1, 1, file) != 1) 
return error("cannot read version\n"); 
if (version != 224) 
return error("version != 224\n"); 

int date; // fixme: really a date? isn't it always zero? 
if (fread(&date, 4, 1, file) != 1) 
return error("cannot read date\n"); 

unsigned char file_id[16]; 
if (fread(&file_id, 1, 16, file) != 16) 
return error("cannot read file id\n"); 

int i; 
printf("file id ="); 
for (i=0; i<16; i++) 
printf(" %02x", (int)file_id[i]); 
printf("\n"); 

short number_partial_hashes; 
if (fread(&number_partial_hashes, 2, 1, file) != 1) 
return error("cannot read number of partial hashes\n"); 
printf("number of partial hashes = %d\n", number_partial_hashes); 

for (int hash_num = 0; hash_num < number_partial_hashes; hash_num++) { 
unsigned char hash[16]; 

if (fread(&hash, 1, 16, file) != 16) 
return error("cannot read hash %d\n", hash_num); 

int i; 
printf("hash %3d =", hash_num); 
for (i=0; i<16; i++) 
printf(" %02x", (int)hash[i]); 
printf("\n"); 
} 

int num_meta_tags; 
if (fread(&num_meta_tags, 4, 1, file) != 1) 
return error("cannot read number of number of meta tags\n"); 

printf("\n"); 
printf("number of meta tags = %d\n", num_meta_tags); 

int gap_start, total_missing = 0; 
int first_gap = 0x7fffffff; 

for (int tag_num = 0; tag_num < num_meta_tags; tag_num++) { 
unsigned char tag_type; 
if (fread(&tag_type, 1, 1, file) != 1) 
return error("cannot read tag type\n"); 

short name_length; 
if (fread(&name_length, 2, 1, file) != 1) 
return error("cannot read name length\n"); 

char name[4096]; 
if (fread(&name, 1, name_length, file) != (size_t) name_length) 
return error("cannot read name\n"); 
name[name_length] = '\0'; 

if (tag_type == 2) { 
short value_length; 
if (fread(&value_length, 2, 1, file) != 1) 
return error("cannot read value length\n"); 

char value[4096]; 
if (fread(&value, 1, value_length, file) != (size_t) value_length) 
return error("cannot read value\n"); 
value[value_length] = '\0'; 

if (name_length == 1) { 
switch (name[0]) { 
case 1: 
printf("filename = \"%s\"\n", value); 
break; 
case 3: 
printf("file type = \"%s\"\n", value); 
break; 
case 4: 
printf("file format = \"%s\"\n", value); 
break; 
case 18: 
printf("tempfile name = \"%s\"\n", value); 
break; 
default: 
printf("tag number %02d = \"%s\"\n", name[0], value); 
break; 
} 
} 
else 
printf("%-13s = \"%s\"\n", name, value); 
} 
else if (tag_type == 3) { 
int value; 
if (fread(&value, 4, 1, file) != 1) 
return error("cannot read value\n"); 

if (name_length == 1) { 
switch (name[0]) { 
case 2: 
printf("file size = %d (%.2f mb)\n", value, (double)value/(1024*1024)); 
break; 
case 8: 
printf("copied so far = %d (%.2f mb)\n", value, (double)value/(1024*1024)); 
break; 
case 19: 
printf("priority = %s\n", priority(value)); 
break; 
case 20: 
printf("status = %s\n", status(value)); 
break; 
default: 
printf("tag number %02d = %d\n", name[0], value); 
break; 
} 
} 
else { 
if (name[0] == 9) { 
printf("gap %3s from %10d", &name[1], value); 
gap_start = value; 
if (gap_start < first_gap) 
first_gap = gap_start; 
} 
else if (name[0] == 10) { 
int missing = value - gap_start; 
total_missing += missing; 
printf(" to %10d = %10d bytes missing\n", value, missing); 
} 
else 
printf("%-13s = %d\n", name, value); 
} 
} 
else 
return error("unknown tag type %d\n", tag_type); 
} 

printf("%d bytes = %.2f mb missing\n", total_missing, (double)total_missing/(1024*1024)); 
if (first_gap < 0x7fffffff) 
printf("first gap starts at %d (%d blocks are complete)\n", first_gap, first_gap/(9500*1024)); 

return true; 
} 

bool partInfo(const char* filename) { 
FILE* file = fopen(filename, "rb"); 
if (file == 0) 
return error("cannot open %s\n", filename); 

bool result = partInfo(file); 
fclose(file); 
return result; 
} 

int main(int argc, const char* argv[]) { 
if (argc != 2) { 
error("usage: %s <number of .part.met file>\n", argv[0]); 
return 0; 
} 

const char* temppath = EDONKEYTEMPPATH; 
char filename[4096]; 
sprintf(filename, "%s%s.part.met", temppath, argv[1]); 

if (!partInfo(filename)) 
error("sorry, cannot retrieve information about %s\n", filename); 

return 0; 
} 
